package com.chelvc.cloud.vehicle.client;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.HelpCategoryDTO;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryPagingParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 帮助分类Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@FeignClient("vehicle")
public interface HelpCategoryClient {
    /**
     * 查询帮助分类
     *
     * @param id 帮助分类主键
     * @return 帮助分类
     */
    @GetMapping("/helpCategory/selectHelpCategoryById")
    HelpCategoryDTO selectHelpCategoryById(@RequestParam("id") Long id);

    /**
     * 查询帮助分类列表
     *
     * @param helpCategory 帮助分类
     * @return 帮助分类集合
     */
    @PostMapping("/helpCategory/selectHelpCategoryList")
    List<HelpCategoryDTO> selectHelpCategoryList(@SpringQueryMap HelpCategoryModifyParam helpCategory);

    /**
     * 新增帮助分类
     *
     * @param helpCategory 帮助分类
     * @return 结果
     */
    @PostMapping("/helpCategory/insertHelpCategory")
    void insertHelpCategory(@RequestBody HelpCategoryModifyParam helpCategory);

    /**
     * 修改帮助分类
     *
     * @param helpCategory 帮助分类
     * @return 结果
     */
    @PutMapping("/helpCategory/updateHelpCategory")
    void updateHelpCategory(@RequestParam("id") Long id, @RequestBody HelpCategoryModifyParam helpCategory);


    /**
     * 删除帮助分类信息
     *
     * @param id 帮助分类主键
     * @return 结果
     */
    @DeleteMapping("/helpCategory")
    void deleteHelpCategoryById(@RequestParam("id") Long id);

    /**
     * 分页查询帮助分类列表
     *
     * @param param
     * @return
     */
    @GetMapping("/helpCategory/getHelpCategoryPaging")
    Pagination<HelpCategoryDTO> getHelpCategoryPaging(@SpringQueryMap HelpCategoryPagingParam param);


    /**
     * 查询帮助分类列表
     *
     * @return 帮助分类集合
     */
    @GetMapping("/helpCategory/selectHelpCategories")
    List<HelpCategoryDTO> selectHelpCategories();
}
