package com.chelvc.cloud.vehicle.client;

import com.chelvc.cloud.vehicle.client.dto.OrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyPagingParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 订单退货申请业务接口
 *
 * @author liude
 * @data 2024/05/01
 */
@FeignClient("vehicle")
public interface OrderReturnApplyClient {

    /**
     * 新增订单退货申请
     * @param param
     * @return
     */
    @PostMapping("/omsOrderReturnApply/add")
    Long add(@SpringQueryMap OrderReturnApplyModifyParam param);

    /**
     * 修改订单退货申请
     * @param param
     * @return
     */
    @PutMapping("/omsOrderReturnApply/update")
    Long update(@RequestParam("id") Long id,@SpringQueryMap OrderReturnApplyModifyParam param);

    /**
     * 订单退货申请分页查询
     * @param param
     * @return
     */
    @GetMapping("/omsOrderReturnApply/paging")
    Pagination<OrderReturnApplyDTO> paging(@SpringQueryMap OrderReturnApplyPagingParam param);



    /**
     * 订单退货申请查询详情
     * @param id
     * @return
     */
    @GetMapping("/omsOrderReturnApply/getDetail")
    OrderReturnApplyDTO getDetail(@RequestParam("id")Long id);

    /**
     * 删除订单退货申请
     * @param id
     * @return
     */
    @GetMapping("/omsOrderReturnApply/delete")
    void delete(@RequestParam("id")Long id);

}
