package com.chelvc.cloud.vehicle.client.util;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.upay.sdk.Constants;
import com.upay.sdk.HttpClientUtils;
import com.upay.sdk.core.CipherWrapper;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.Executer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MyExecuter extends Executer {
    static final Logger LOGGER = LoggerFactory.getLogger(MyExecuter.class);

    public JSONObject ApiRequest(JsonBuilder jsonBuilder, String apiUrl) {
        try {
            JSONObject requestData = jsonBuilder.buildRequestParam();

            LOGGER.debug("API requestData:[" + requestData.toJSONString() + "]");
            //接口同步应答报文
            String responseStr = HttpClientUtils.post3(apiUrl, requestData);
            LOGGER.info("API responseStr :{}", responseStr);

            JSONObject responseData = JSONObject.parseObject(responseStr, new Feature[]{Feature.SortFeidFastMatch});
            if (Constants.ERROR.equals(responseData.getString(Constants.STATUS))) {
                //如果接口直接返回ERROR时，不需要解密
                return responseData;
            }

            //解密
            responseData = CipherWrapper.bothDecryptWrap(responseData);
            if (Constants.ERROR.equals(responseData.getString(Constants.STATUS))) {
                //解密后报文，如果status=ERROR时，不需要验签
                return responseData;
            }

            //验签
            super.bothVerifyHmacOrder(responseData);

            LOGGER.info("API responseData :{}", responseStr);
            return responseData;

        } catch (Exception exception) {
            LOGGER.error("API request exception", exception);
            throw new UnknownException(exception);
        }
    }

    public JSONObject getDecrypt(String data){
        JSONObject responseData = JSONObject.parseObject(data, new Feature[]{Feature.SortFeidFastMatch});
        if (Constants.ERROR.equals(responseData.getString(Constants.STATUS))) {
            //如果接口直接返回ERROR时，不需要解密
            return responseData;
        }
        //解密
        responseData = CipherWrapper.bothDecryptWrap(responseData);
        if (Constants.ERROR.equals(responseData.getString(Constants.STATUS))) {
            //解密后报文，如果status=ERROR时，不需要验签
            return responseData;
        }
        //验签
        super.bothVerifyHmacOrder(responseData);

        LOGGER.info("API responseData :{}", data);
        return responseData;
    }
}
