package com.chelvc.cloud.vehicle.server.controller;

import com.chelvc.cloud.vehicle.client.dto.AssessMessageDto;
import com.chelvc.cloud.vehicle.client.dto.GoodsAssessDto;
import com.chelvc.cloud.vehicle.client.param.AssessModifyParam;
import com.chelvc.cloud.vehicle.client.param.AssessQueryParam;
import com.chelvc.cloud.vehicle.client.param.AssessRecoverModifyParam;
import com.chelvc.cloud.vehicle.server.service.AssessService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @author Woody
 * @date 2024/4/21
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AssessController {
    private final AssessService assessService;

    @PostMapping("/assess/add")
    public Long add(@RequestBody AssessModifyParam param) {
        return this.assessService.add(param);
    }

    @GetMapping("/assess/query")
    public AssessMessageDto query(@SpringQueryMap AssessQueryParam param) {
        return this.assessService.query(param);
    }

    @GetMapping("/assess/detail")
    public GoodsAssessDto detail(@RequestParam("id") Long assessId) {
        return this.assessService.getDetail(assessId);
    }

    @PostMapping("/assess/merchantQuery")
    public List<GoodsAssessDto> merchantQuery(@SpringQueryMap AssessQueryParam param) {
        return this.assessService.merchantQuery(param);
    }

    @PostMapping("/assess/addAssessRecover")
    public Long addAssessRecover(@RequestBody AssessRecoverModifyParam param) {
        return this.assessService.addAssessRecover(param);
    }

    @PostMapping("/assess/computeAvgScore")
    public void computeAvgScore() {
        this.assessService.computeAvgScore();
    }

    @GetMapping("/assess/getGoodsScore")
    public Map<String, BigDecimal> getGoodsScore(@RequestParam("goodsId") Long goodsId) {
        return this.assessService.getGoodsScore(goodsId);
    }
}
