package com.chelvc.cloud.vehicle.server.controller;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.client.param.CategoryListParam;
import com.chelvc.cloud.vehicle.client.param.CategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.CategoryPagingParam;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 分类业务操作接口
 *
 * @author Woody
 * @date 2023/5/2
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CategoryController {

    private final CategoryService categoryService;

    /**
     * 新增分类
     *
     * @param param 新增参数
     * @return 分类主键
     */
    @PostMapping("/category/addCategory")
    public Long addCategory(@RequestBody CategoryModifyParam param) {
        return categoryService.addCategory(param);
    }

    /**
     * 修改分类
     *
     * @param id    分类主键
     * @param param 修改参数
     */
    @PutMapping("/category/updateCategory")
    public void updateCategory(@RequestParam("id") Long id, @RequestBody CategoryModifyParam param) {
        categoryService.updateCategory(id, param);
    }

    /**
     * 查询分类
     *
     * @param param 查询参数
     * @return 分类分页信息
     */
    @GetMapping("/category/getCategoryPaging")
    public Pagination<CategoryDTO> getCategoryPaging(CategoryPagingParam param) {
        return categoryService.getCategoryPaging(param);
    }

    /**
     * 获取有效分类信息
     *
     * @return 分类信息列表
     */
    @GetMapping("/category/listActiveCategories")
    public List<CategoryDTO> listActiveCategories(CategoryListParam param) {
        return categoryService.listActiveCategories(param);
    }

    /**
     * 获取有效分类信息
     *
     * @return 分类信息列表
     */
    @GetMapping("/category/listCategory")
    public List<CategoryDTO> listCategory(CategoryListParam param) {
        return categoryService.listCategory(param);
    }

    /**
     * 获取推荐分类信息
     *
     * @param size 分类个数
     * @return 分类信息列表
     */
    @GetMapping("/category/listRecommendCategories")
    public List<CategoryDTO> listRecommendCategories(@RequestParam("size") int size) {
        return categoryService.listRecommendCategories(size);
    }


    /**
     * 删除分类
     *
     * @param id 分类主键
     */
    @DeleteMapping("/category")
    public void deleteCategory(@RequestParam("id") Long id) {
        categoryService.deleteCategory(id);
    }

    /**
     * 获取分类列表
     *
     * @return 分类信息
     */
    @GetMapping("/category/getRootCategories")
    public List<CategoryDTO> getRootCategories(CategoryListParam param) {
        return categoryService.getRootCategories(param);
    }

    /**
     * 获取一级分类
     *
     * @return 分类信息
     */
    @GetMapping("/category/getParentCategories")
    public List<CategoryDTO> getParentCategories(CategoryListParam param) {
        return categoryService.getParentCategories(param);
    }
}
