package com.chelvc.cloud.vehicle.server.controller;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.NoticeDTO;
import com.chelvc.cloud.vehicle.client.param.NoticeModifyParam;
import com.chelvc.cloud.vehicle.client.param.NoticePagingParam;
import com.chelvc.cloud.vehicle.server.service.NoticeService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 通知配置业务接口
 *
 * @author liude
 * @data 2023/12/7
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class NoticeController {

    public final NoticeService noticeService;

    /**
     * 新增通知
     *
     * @param param 新增参数
     * @return 主键
     */
    @PostMapping("/notice/addNotice")
    public Long addNotice(@RequestBody NoticeModifyParam param) {
        return noticeService.addNotice(param);
    }

    /**
     * 修改通知
     *
     * @param id    主键
     * @param param 修改参数
     */
    @PutMapping("/notice/updateNotice")
    public void updateNotice(@RequestParam("id") Long id, @RequestBody NoticeModifyParam param) {
        noticeService.updateNotice(id, param);
    }

    /**
     * 查询通知
     *
     * @param param 查询参数
     * @return 通知分页信息
     */
    @GetMapping("/notice/getNoticePaging")
    public Pagination<NoticeDTO> getNoticePaging(NoticePagingParam param) {
        return noticeService.getNoticePaging(param);
    }

    /**
     * 查询通知列表
     *
     * @return 通知信息
     */
    @GetMapping("/notice/getNoticeList")
    public List<NoticeDTO> getNoticeList(@RequestParam("type") String type) {
        return noticeService.getNoticeList(type);
    }
}
