package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.chelvc.cloud.vehicle.client.constant.PlatformConstant;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantRelation;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.StandardCharsets;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Objects;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OderHandleServiceImpl implements OrderHandleService {
    private final OmsOrderMapper omsOrderMapper;
    private final UserInviteService userInviteService;
    private final UserProfitRatioService userProfitRatioService;
    private final PlatformProfitRatioService platformProfitRatioService;
    private final AssetService assetService;
    private final BalanceDetailService balanceDetailService;
    private final OmsOrderOperateHistoryService omsOrderOperateHistoryService;
    private final MerchantRelationService merchantRelationService;

    @Override
    public void orderPay(Long orderId) {
        LambdaQueryWrapper<OmsOrder> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(OmsOrder::getId, orderId);
        wrapper.eq(OmsOrder::getStatus, 0);
        OmsOrder omsOrder = omsOrderMapper.selectOne(wrapper);
        if (omsOrder == null) {
            return;
        }
        StringBuilder sb = new StringBuilder();
        sb.append("merchantAcctId=").append("1001292117101").append("&");
        sb.append("version=").append("v2.0").append("&");
        sb.append("language=").append("1").append("&");
        sb.append("signType=").append("4").append("&");
        sb.append("payType=").append("21").append("&");
        sb.append("bankId=").append("QTYH").append("&");
        sb.append("orderId=").append("KQ20220609155005").append("&");
        sb.append("orderTime=").append("20220609155005").append("&");
        sb.append("orderAmount=").append("1").append("&");
        sb.append("bindCard=").append("6222027781").append("&");
        sb.append("bindMobile=").append("1662653").append("&");
        sb.append("dealId=").append("104964327").append("&");
        sb.append("bankDealId=").append("9841654761026121342").append("&");
        sb.append("dealTime=").append("20220609155546").append("&");
        sb.append("payAmount=").append("1").append("&");
        sb.append("fee=").append("0").append("&");
        sb.append("payResult=").append("10");
    }

    @Override
    public void handleOrder(OmsOrder omsOrder) {
        omsOrder.setStatus(3);
        omsOrder.setConfirmStatus(1);
        omsOrder.setReceiveTime(new Date());
        omsOrder.setUpdateTime(new Date());
        int i = omsOrderMapper.updateById(omsOrder);
        if (i != 1) {
            throw new ResourceUnavailableException("处理失败");
        }
        Long orderId = omsOrder.getId();
        //分佣逻辑=====================================
        //实际支付金额
        BigDecimal payAmount = omsOrder.getPayAmount();
        Long userId = omsOrder.getUserId();
        Long merchantId = omsOrder.getMerchantId();
        BigDecimal userAmount = BigDecimal.ZERO;
        BigDecimal merchantAmount = BigDecimal.ZERO;
        BigDecimal operate = payAmount.subtract(omsOrder.getFeeAmount());
        //平台金额扣除商品金额
        BigDecimal bigDecimal = assetService.updateAsset(1, operate, 0L, 0, 1);
        balanceDetailService.recordFlow(orderId, 0, 0L, operate, BigDecimal.ZERO,
                operate, bigDecimal, 1, 6, userId);
        //平台抽佣
        int platformRatio = platformProfitRatioService.queryRatio(merchantId);
        //平台抽佣金额----向上取整保留2位小数
        BigDecimal platformAmount = new BigDecimal(platformRatio).divide(new BigDecimal("100"), 2, RoundingMode.UP)
                .multiply(payAmount).setScale(2, RoundingMode.UP);
        //用户抽佣----查看消费用户上级
        Long inviteUserId = userInviteService.queryUserByTarget(userId, 1);
        if (inviteUserId != null) {
            //用户分佣
            int userRatio = userProfitRatioService.queryRatio(inviteUserId, 1);
            if (userRatio > 0) {
                userAmount = new BigDecimal(userRatio).divide(new BigDecimal("100"), 2, RoundingMode.UP)
                        .multiply(platformAmount).setScale(2, RoundingMode.UP);
            }
        }
        //商户抽佣----查看消费用户上级
        Long inviteMerchantId = userInviteService.queryUserByTarget(merchantId, 2);
        if (inviteMerchantId != null) {
            //商户分佣
            int merchantRatio = userProfitRatioService.queryRatio(inviteMerchantId, 2);
            if (merchantRatio > 0) {
                merchantAmount = new BigDecimal(merchantRatio).divide(new BigDecimal("100"), 2, RoundingMode.UP)
                        .multiply(platformAmount).setScale(2, RoundingMode.UP);
            }
        }
        //商家应得金额
        BigDecimal operateAmount = payAmount.subtract(platformAmount);
        //记录流水--------------------------
        //商家流水
        BigDecimal merchantDeduct = BigDecimal.ZERO;  //扣税
        BigDecimal merchantRealityAmount = operateAmount.subtract(merchantDeduct);
        BigDecimal surplusAmount = assetService.updateAsset(0, merchantRealityAmount, merchantId, 2, 1);
        //currencyRecordService.recordFlow(merchantId, 2, 0, orderId, userId, 0, operateAmount, merchantDeduct, merchantRealityAmount);
        //商品出售商家记录余额明细
        balanceDetailService.recordFlow(orderId, 2, merchantId, operateAmount, merchantDeduct,
                merchantRealityAmount, surplusAmount, 0, 0, userId);
        //给商家转账
        assetService.transfer(0L, merchantId, 2, merchantRealityAmount, "");
        //邀请用户---分得佣金
        if (userAmount.compareTo(BigDecimal.ZERO) > 0) {
            platformAmount = platformAmount.subtract(userAmount);
            BigDecimal deduct = BigDecimal.ZERO;  //扣税
            BigDecimal realityAmount = userAmount.subtract(deduct);
            MerchantRelation relation = merchantRelationService.queryByUserIdAndType(inviteUserId, 1);
            if(relation != null && StringUtils.isNotBlank(relation.getSxyMerchantId())){
                //开通商户号，转账操作
                assetService.transfer(0L, inviteUserId, 1, realityAmount, "");
                //记录用户余额明细
                balanceDetailService.recordFlow(orderId, 1, inviteUserId, userAmount, deduct,
                        realityAmount, surplusAmount, 0, 1, userId);
            } else {
                //更新用户资产
                BigDecimal userTotal = assetService.updateAsset(0, realityAmount, inviteUserId, 1, 1);
                //邀请用户余额明细
                balanceDetailService.recordFlow(orderId, 1, inviteUserId, userAmount, deduct,
                        realityAmount, userTotal, 0, 1, userId);
                //用户资产记录到平台名下
                BigDecimal platformTotal = assetService.updateAsset(0, realityAmount, 0L, 0, 1);
                //用户佣金记录到平台余额名下
                balanceDetailService.recordFlow(orderId, 0, 0L, userAmount, deduct,
                        realityAmount, platformTotal, 0, 1, userId);
            }
        }
        //邀请商家---分得佣金
        if (merchantAmount.compareTo(BigDecimal.ZERO) > 0) {
            platformAmount = platformAmount.subtract(merchantAmount);
            BigDecimal deduct = BigDecimal.ZERO;  //扣税
            BigDecimal realityAmount = merchantAmount.subtract(deduct);
            //更新商家资产
            BigDecimal merchantTotal = assetService.updateAsset(0, realityAmount, inviteMerchantId, 1, 1);
            //记录商户分佣余额明细
            balanceDetailService.recordFlow(omsOrder.getId(), 2, inviteMerchantId, merchantAmount, deduct,
                    realityAmount, merchantTotal, 0, 2, userId);
            //商家佣金转账
            assetService.transfer(0L, inviteMerchantId, 2, realityAmount, "");
        }
        //平台流水
        BigDecimal deduct = BigDecimal.ZERO;  //扣税
        BigDecimal platformRealityAmount = platformAmount.subtract(deduct);
        //更新平台资产
        BigDecimal platformTotal = assetService.updateAsset(0, platformRealityAmount, 0L, 0, 1);
        //记录平台抽成余额明细
        balanceDetailService.recordFlow(omsOrder.getId(), 0, 0L, platformAmount, deduct,
                platformRealityAmount, platformTotal, 0, 2, userId);
        omsOrderOperateHistoryService.insertOmsOrderOperateHistory(omsOrder.getId(), userId, omsOrder.getStatus());
        //以下注释为以前逻辑，先不用管
        /*BigDecimal platformSurplusAmount = assetService.updateAsset(1, operateAmount, 0L, 0);
        balanceDetailService.recordFlow(omsOrder.getId(), 0, 0L, operateAmount, platformSurplusAmount, 1, 1,
                merchantId);
        //邀请用户---分得佣金
        if (userAmount.compareTo(BigDecimal.ZERO) > 0) {
            BigDecimal userSurplusAmount = assetService.updateAsset(0, userAmount, inviteUserId, 1);
            balanceDetailService.recordFlow(omsOrder.getId(), 1, inviteUserId, userAmount, userSurplusAmount, 0, 1,
                    userId);
            BigDecimal platformSurplusAmount1 = assetService.updateAsset(1, userAmount, 0L, 0);
            balanceDetailService.recordFlow(omsOrder.getId(), 0, 0L, userAmount, platformSurplusAmount1, 1, 1,
                    inviteUserId);
        }
        //邀请商家---分得佣金
        if (merchantAmount.compareTo(BigDecimal.ZERO) > 0) {
            BigDecimal merchantSurplusAmount2 = assetService.updateAsset(0, merchantAmount, inviteMerchantId, 1);
            balanceDetailService.recordFlow(omsOrder.getId(), 1, inviteMerchantId, merchantAmount,
                    merchantSurplusAmount2, 0, 1, merchantId);
            BigDecimal platformSurplusAmount2 = assetService.updateAsset(1, merchantAmount, 0L, 0);
            balanceDetailService.recordFlow(omsOrder.getId(), 0, 0L, merchantAmount, platformSurplusAmount2, 1, 1,
                    inviteMerchantId);
        }*/
    }

    @Override
    public void paySuccess(OmsOrder order) {
        omsOrderMapper.updateById(order);
        assetService.updateAsset(0, order.getPayAmount(), PlatformConstant.PLATFORM_ID, 0, 2);
        omsOrderOperateHistoryService.insertOmsOrderOperateHistory(order.getId(), order.getUserId(), order.getStatus());
       /* BigDecimal surplusAmount = assetService.updateAsset(0, order.getPayAmount(), 0L, 0);
        balanceDetailService.recordFlow(order.getId(), 0, 0L, order.getPayAmount(), surplusAmount, 0, 1,
                order.getUserId());*/
    }

    public static void main(String[] args) {

        StringBuilder sb = new StringBuilder();
        sb.append("merchantAcctId=").append("1001292117101").append("&");
        sb.append("version=").append("v2.0").append("&");
        sb.append("language=").append("1").append("&");
        sb.append("signType=").append("4").append("&");
        sb.append("payType=").append("21").append("&");
        sb.append("bankId=").append("QTYH").append("&");
        sb.append("orderId=").append("KQ20220609155005").append("&");
        sb.append("orderTime=").append("20220609155005").append("&");
        sb.append("orderAmount=").append("1").append("&");
        sb.append("bindCard=").append("6222027781").append("&");
        sb.append("bindMobile=").append("1662653").append("&");
        sb.append("dealId=").append("104964327").append("&");
        sb.append("bankDealId=").append("9841654761026121342").append("&");
        sb.append("dealTime=").append("20220609155546").append("&");
        sb.append("payAmount=").append("1").append("&");
        sb.append("fee=").append("0").append("&");
        sb.append("payResult=").append("10");
        String base64 = "";
        String SignMsgVal = "merchantAcctId=1001292117101&version=v2" +
                ".0&language=1&signType=4&payType=21&bankId=QTYH&orderId=KQ20220609155005&orderTime=20220609155005" +
                "&orderAmount=1&bindCard=6222027781&bindMobile=1662653&dealId=104964327&bankDealId" +
                "=9841654761026121342&dealTime=20220609155546&payAmount=1&fee=0&payResult=10";

        //人民币网关账号，该账号为11位人民币网关商户编号+01,该参数必填。
        String merchantAcctId = "1001295168301";
        //编码方式，1代表 UTF-8; 2 代表 GBK; 3代表 GB2312 默认为1,该参数必填。
        String inputCharset = "1";
        //接收支付结果的页面地址，该参数一般置为空即可。
        String pageUrl = "";
        //服务器接收支付结果的后台地址，该参数务必填写，不能为空。
        String bgUrl = "http://www.99bill.com/";
        //网关版本，固定值：mobile1.0,该参数必填。
        String version = "mobile1.0";
        //移动网关版本 phone代表手机版移动网关，pad代表平板移动网关，默认为phone
        String mobileGateway = "";
        //语言种类，1代表中文显示，2代表英文显示。默认为1,该参数必填。
        String language = "1";
        //签名类型,该值为4，代表PKI加密方式,该参数必填。
        String signType = "4";
        //支付人姓名,可以为空。
        String payerName = "";
        //支付人联系类型，1 代表电子邮件方式；2 代表手机联系方式。可以为空。
        String payerContactType = "";
        //支付人联系方式，与payerContactType设置对应，payerContactType为1，则填写邮箱地址；payerContactType为2，则填写手机号码。可以为空。
        String payerContact = "";
        //指定付款人，可以为空
        String payerIdType = "";
        //付款人标识，可以为空
        String payerId = "";
        //付款人IP，可以为空
        String payerIP = "";
        //商家的终端ip，支持Ipv4和Ipv6
        String terminalIp = "192.168.1.1";
        //网络交易平台简称，英文或中文字符串,除微信支付宝支付外其他交易方式必传
        String tdpformName = "";
        //商户订单号，以下采用时间来定义订单号，商户可以根据自己订单号的定义规则来定义该值，不能为空。
        String orderId = "KQ" + new java.text.SimpleDateFormat("yyyyMMddHHmmss").format(new java.util.Date());
        //订单金额，金额以“分”为单位，商户测试以1分测试即可，切勿以大金额测试。该参数必填。
        String orderAmount = "1";
        //订单提交时间，格式：yyyyMMddHHmmss，如：20071117020101，不能为空。
        String orderTime = new java.text.SimpleDateFormat("yyyyMMddHHmmss").format(new java.util.Date());
        //String orderTime = "20220224145101";
        //快钱时间戳，格式：yyyyMMddHHmmss，如：20071117020101， 可以为空
        String orderTimestamp = "";//new java.text.SimpleDateFormat("yyyyMMddHHmmss").format(new java.util.Date());;
        //商品名称，可以为空。
        String productName = "测试商品";
        //商品数量，可以为空。
        String productNum = "1";
        //商品代码，可以为空。
        String productId = "";
        //商品描述，可以为空。
        String productDesc = "";
        //扩展字段1，商户可以传递自己需要的参数，支付完快钱会原值返回，可以为空。
        String ext1 = "";
        //扩展自段2，商户可以传递自己需要的参数，支付完快钱会原值返回，可以为空。
        String ext2 = "";
        //支付方式，一般为00，代表所有的支付方式。微信小程序26-3
        String payType = "26-3";
        //银行代码，如果payType为00，该值可以为空
        String bankId = "";
        //同一订单禁止重复提交标志，实物购物车填1，虚拟产品用0。1代表只能提交一次，0代表在支付不成功情况下可以再提交。可为空。
        String redoFlag = "0";
        //快钱合作伙伴的帐户号，即商户编号，可为空。
        String pid = "";
        //订单超时时间(秒)，可为空。
        String orderTimeOut = "";
        //聚合支付参数
        String aggregatePay = "26-3:[appId=wx44377c7a02caaac9,openId=oxr7946y03Ui9LjsR4zGFi8OssbY,limitPay=0]";

        //signMsg 签名字符串 不可空，生成加密签名串
        String signMsgVal = "";
        signMsgVal = appendParam(signMsgVal, "inputCharset", inputCharset);
        signMsgVal = appendParam(signMsgVal, "pageUrl", pageUrl);
        signMsgVal = appendParam(signMsgVal, "bgUrl", bgUrl);
        signMsgVal = appendParam(signMsgVal, "version", version);
        signMsgVal = appendParam(signMsgVal, "language", language);
        signMsgVal = appendParam(signMsgVal, "signType", signType);
        signMsgVal = appendParam(signMsgVal, "merchantAcctId", merchantAcctId);
        signMsgVal = appendParam(signMsgVal, "payerName", payerName);
        signMsgVal = appendParam(signMsgVal, "payerContactType", payerContactType);
        signMsgVal = appendParam(signMsgVal, "payerContact", payerContact);
        signMsgVal = appendParam(signMsgVal, "payerIdType", payerIdType);
        signMsgVal = appendParam(signMsgVal, "payerId", payerId);
        signMsgVal = appendParam(signMsgVal, "payerIP", payerIP);
        signMsgVal = appendParam(signMsgVal, "orderId", orderId);
        signMsgVal = appendParam(signMsgVal, "orderAmount", orderAmount);
        signMsgVal = appendParam(signMsgVal, "orderTime", orderTime);
        signMsgVal = appendParam(signMsgVal, "orderTimestamp", orderTimestamp);
        signMsgVal = appendParam(signMsgVal, "productName", productName);
        signMsgVal = appendParam(signMsgVal, "productNum", productNum);
        signMsgVal = appendParam(signMsgVal, "productId", productId);
        signMsgVal = appendParam(signMsgVal, "productDesc", productDesc);
        signMsgVal = appendParam(signMsgVal, "ext1", ext1);
        signMsgVal = appendParam(signMsgVal, "ext2", ext2);
        signMsgVal = appendParam(signMsgVal, "payType", payType);
        signMsgVal = appendParam(signMsgVal, "bankId", bankId);
        signMsgVal = appendParam(signMsgVal, "redoFlag", redoFlag);
        signMsgVal = appendParam(signMsgVal, "pid", pid);
        signMsgVal = appendParam(signMsgVal, "orderTimeOut", orderTimeOut);
        signMsgVal = appendParam(signMsgVal, "mobileGateway", mobileGateway);
        signMsgVal = appendParam(signMsgVal, "aggregatePay", aggregatePay);

        try {
            System.out.println(signMsgVal);
            KeyStore ks = KeyStore.getInstance("PKCS12");

            //读取密钥仓库（相对路径）
            String file = Objects.requireNonNull(OderHandleServiceImpl.class.getClassLoader().getResource("payUtils" +
                    "/10012951683.pfx")).getPath().replaceAll("%20", " ");
            System.out.println(file);

            FileInputStream ksfis = new FileInputStream(file);
            //InputStream inputStream = PayServiceImpl.class.getClassLoader().getResourceAsStream
            // ("payUtils/10012951683.pfx");
            BufferedInputStream ksbufin = new BufferedInputStream(ksfis);
            //证书密码
            char[] keyPwd = "123456".toCharArray();
            ks.load(ksbufin, keyPwd);

            //证书别名	20190802.3300000002967055(10012155863)、20190801.3300000002925831(10012921171)
            PrivateKey priK = (PrivateKey) ks.getKey("test-alias", keyPwd); //老别名：test-alias
            //签名算法
            Signature signature = Signature.getInstance("SHA256withRSA");  //SHA256withRSA，SHA1withRSA
            signature.initSign(priK);
            //signature.update(SignMsgVal.getBytes(StandardCharsets.UTF_8));
            signature.update(signMsgVal.getBytes(StandardCharsets.UTF_8));
            sun.misc.BASE64Encoder encoder = new sun.misc.BASE64Encoder();
            base64 = encoder.encode(signature.sign());

        } catch (FileNotFoundException e) {
            System.out.println("123");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        System.out.println("test = " + base64);
        String signMsg = "Yku7O/TyRnCErbGwvo/EQxgNLgR61YJ9WQrxpD0I6HxIumGVRxQo4eQ/hn5CS57xM2GnJ7qtkWM3J" +
                "/g9CRcK22S7EJ4E1zK2HuqYiScaFLolHWNszGe7kOPk3PovTFl4nGYUSW8vMzZEZgRTMdHNSpmib57o6Mrjq0" +
                "+6xA1gSBBRqpaL6SO4IBfB6yldWHpkJN0RJlKlkGGRJjMEpLJPn8lTGPcOfJuNxmMq+qjbvdLyoxn7v0vK8bAPFT45bCdvGG" +
                "+0HwUCYxtPPhFReZMXP/yJYEZ/TODSMlPhGM1dZXXbnZ5Sc4KJSWNB9CKgyKWcycMbfTlsraWtpsCwOr8k0g==";
        try {
            //快钱公钥
            //String file = Pkipair.class.getResource("99bill.RSA.cer").toURI().getPath();
            String file = Objects.requireNonNull(OderHandleServiceImpl.class.getClassLoader().getResource("payUtils" +
                    "/CFCA_sandbox.cer")).toURI().getPath();
            //System.out.println(file);
            FileInputStream inStream = new FileInputStream(file);

            CertificateFactory cf = CertificateFactory.getInstance("X.509");
            X509Certificate cert = (X509Certificate) cf.generateCertificate(inStream);

            PublicKey pk = cert.getPublicKey();
            //验签算法
            Signature signature = Signature.getInstance("SHA256withRSA");//SHA256withRSA，SHA1withRSA
            signature.initVerify(pk);
            signature.update(signMsgVal.getBytes());

            sun.misc.BASE64Decoder decoder = new sun.misc.BASE64Decoder();
            //System.out.println(new String(decoder.decodeBuffer(msg)));
            boolean flag = signature.verify(decoder.decodeBuffer(base64));
            System.out.println(flag);
        } catch (Exception e) {
            e.printStackTrace();
            //System.out.println("no");
        }
    }

    public static String appendParam(String returns, String paramId, String paramValue) {
        if (!Objects.equals(returns, "")) {
            if (!Objects.equals(paramValue, "")) {
                returns += "&" + paramId + "=" + paramValue;
            }
        } else {
            if (!Objects.equals(paramValue, "")) {
                returns = paramId + "=" + paramValue;
            }
        }
        return returns;
    }
}
