package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.server.dao.OmsTransferWithdrawMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsTransferWithdraw;
import com.chelvc.cloud.vehicle.server.service.OmsTransferWithdrawService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired,@Lazy}))
public class OmsTransferWithdrawServiceImpl extends ServiceImpl<OmsTransferWithdrawMapper,
        OmsTransferWithdraw> implements OmsTransferWithdrawService {

    @Override
    public Long createTransferWithdraw(Long userId, Integer type, BigDecimal amount, Integer channel, Integer flag) {
        OmsTransferWithdraw transferWithdraw = new OmsTransferWithdraw();
        transferWithdraw.setAmount(amount);
        transferWithdraw.setProcedureAmount(BigDecimal.ZERO);
        transferWithdraw.setChannel(channel);
        transferWithdraw.setStatus(0);
        transferWithdraw.setUserId(userId);
        transferWithdraw.setType(type);
        transferWithdraw.setFlag(flag);
        transferWithdraw.setCreateTime(new Date());
        int i = baseMapper.insert(transferWithdraw);
        if(i != 1){
            throw new ResourceUnavailableException("提现申请失败");
        }
        return transferWithdraw.getId();
    }

    @Override
    public int updateStatusById(Long id, Integer status) {
        UpdateWrapper<OmsTransferWithdraw> update = Wrappers.update();
        update.set("status", status);
        update.eq("id", id);
        return baseMapper.update(null, update);
    }

    @Override
    public int updateStatusById(Long id, Integer status, String remark) {
        UpdateWrapper<OmsTransferWithdraw> update = Wrappers.update();
        update.set("status", status);
        update.set("remark", remark);
        update.set("update_time", new Date());
        update.eq("id", id);
        return baseMapper.update(null, update);
    }

    @Override
    public OmsTransferWithdraw queryById(Long id) {
        return baseMapper.selectById(id);
    }

    @Override
    public int updateByNotify(Long id, String outOrderId, Integer status, String remark) {
        UpdateWrapper<OmsTransferWithdraw> update = Wrappers.update();
        update.set("status", status);
        update.set("out_order_id", outOrderId);
        update.set("update_time", new Date());
        update.set("remark", remark);
        update.eq("id", id);
        return baseMapper.update(null, update);
    }

    @Override
    public int updateByNotify(Long id, String outOrderId, Integer status, String remark, BigDecimal procedureAmount) {
        UpdateWrapper<OmsTransferWithdraw> update = Wrappers.update();
        update.set("status", status);
        update.set("out_order_id", outOrderId);
        update.set("update_time", new Date());
        update.set("remark", remark);
        update.set("procedure_amount", procedureAmount);
        update.eq("id", id);
        return baseMapper.update(null, update);
    }
}
