package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.BaseUserDTO;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.vehicle.client.constant.PlatformConstant;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.dto.TeamDTO;
import com.chelvc.cloud.vehicle.client.dto.TeamDetailsDTO;
import com.chelvc.cloud.vehicle.client.param.TeamInvitePageParam;
import com.chelvc.cloud.vehicle.client.param.UserInviteParams;
import com.chelvc.cloud.vehicle.server.dao.ProfitRatioConfigMapper;
import com.chelvc.cloud.vehicle.server.dao.UserInviteMapper;
import com.chelvc.cloud.vehicle.server.entity.ProfitRatioConfig;
import com.chelvc.cloud.vehicle.server.entity.UserInvite;
import com.chelvc.cloud.vehicle.server.entity.UserProfitRatio;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.cloud.vehicle.server.service.UserInviteService;
import com.chelvc.cloud.vehicle.server.service.UserProfitRatioService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserInviteServiceImpl extends ServiceImpl<UserInviteMapper, UserInvite> implements UserInviteService {

    private final ProfitRatioConfigMapper profitRatioConfigMapper;

    private final UserProfitRatioService userProfitRatioService;

    private final UserClient userService;

    private final MerchantService merchantService;

    @Override
    public void bindInvite(UserInviteParams params) {
        Long userId = params.getUserId();
        Long targetId = params.getTargetId();
        Integer type = params.getType();
        if(userId == null || targetId == null || type == null){
            throw new ResourceUnavailableException("参数错误");
        }
        if(userId.equals(targetId)){
            throw new ResourceUnavailableException("自己不能邀请自己");
        }
        LambdaQueryWrapper<UserInvite> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(UserInvite::getTargetId, targetId);
        wrapper.eq(UserInvite::getType, type);
        Integer count = baseMapper.selectCount(wrapper);
        if(count > 0){
            throw new ResourceUnavailableException("该用户已绑定关系");
        }
        UserInvite userInvite = new UserInvite();
        userInvite.setUserId(userId);
        userInvite.setTargetId(targetId);
        userInvite.setType(type);
        userInvite.setCreateTime(LocalDateTime.now());
        DatabaseContextHolder.transactional((Executor) () -> {
            int i = baseMapper.insert(userInvite);
            if (i < 1) {
                throw new ResourceUnavailableException("绑定失败");
            }
            LambdaQueryWrapper<UserInvite> lwq = Wrappers.lambdaQuery();
            lwq.eq(UserInvite::getUserId, userId);
            lwq.eq(UserInvite::getType, type);
            Integer total = baseMapper.selectCount(lwq);
            int ratio = 0;
            UserProfitRatio userProfitRatio = userProfitRatioService.selectOne(userId, type);
            int oldRatio = userProfitRatio.getRatio();
            LambdaQueryWrapper<ProfitRatioConfig> config = Wrappers.lambdaQuery();
            config.eq(ProfitRatioConfig::getType, type);
            config.eq(ProfitRatioConfig::getStatus, 0);
            config.orderByDesc(ProfitRatioConfig::getNum);
            List<ProfitRatioConfig> list = profitRatioConfigMapper.selectList(config);
            if (!CollectionUtils.isEmpty(list)) {
                for (ProfitRatioConfig profitRatioConfig : list) {
                    int num = profitRatioConfig.getNum();
                    if (total >= num) {
                        ratio = profitRatioConfig.getRatio();
                        break;
                    }
                }
            }
            if (ratio != 0 && ratio != oldRatio) {
                userProfitRatio.setRatio(ratio);
                userProfitRatio.setUpdateTime(LocalDateTime.now());
                userProfitRatio.setUpdater(PlatformConstant.PLATFORM_ID);
                userProfitRatioService.update(userProfitRatio);
            }
        });
    }

    @Override
    public TeamDTO teamList(TeamInvitePageParam param, Long userId) {
        LocalDate today = LocalDate.now(); // 获取今天的日期
        LocalDateTime midnight = LocalDateTime.of(today, LocalTime.MIDNIGHT); // 获取今天凌晨的时间
        LocalDate yesterday = today.minusDays(1);
        LocalDateTime yesterdayMidnight = LocalDateTime.of(yesterday, LocalTime.MIN);
        LocalDateTime toMonth = LocalDateTime.of(LocalDate.from(LocalDateTime.now()
                .with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN);
        LocalDateTime lastMonth = toMonth.minusMonths(1);
        TeamDTO dto = baseMapper.teamTotalStatistics(param.getType(), userId,
                midnight, yesterdayMidnight, toMonth, lastMonth);
        if(dto == null){
            return new TeamDTO();
        }
        BigDecimal totalAmount = dto.getTotalAmount();
        if(totalAmount == null){
            dto.setTotalAmount(BigDecimal.ZERO);
        }
        Paging paging = param.getPaging();
        int pageNum = (paging.getNumber() - 1) * paging.getSize();
        if(pageNum < 0){
            return dto;
        }
        List<TeamDetailsDTO> records = baseMapper.teamList(param.getType(), pageNum, paging.getSize(), userId);
        if(CollectionUtils.isEmpty(records)){
            return dto;
        }
        Map<Long, TeamDetailsDTO> userMap = new HashMap<>();
        if(param.getType() == null){
            List<Long> userIds = records.stream().filter(e -> e.getSourceType() == 1).map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<UserDTO> users = this.userService.listUsers(userIds);
            if(!CollectionUtils.isEmpty(users)){
                users.forEach(e ->{
                    TeamDetailsDTO detailsDTO = new TeamDetailsDTO();
                    detailsDTO.setInviteUserAvatar(e.getAvatar());
                    detailsDTO.setNickname(e.getNickname());
                    detailsDTO.setMobile(e.getMobile());
                    userMap.put(e.getId(), detailsDTO);
                });
            }
            List<Long> merchantIds = records.stream().filter(e -> e.getSourceType() == 2).map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<MerchantDTO> merchants = this.merchantService.getMerchantByIds(merchantIds);
            if(!CollectionUtils.isEmpty(merchants)){
                merchants.forEach(e ->{
                    TeamDetailsDTO detailsDTO = new TeamDetailsDTO();
                    detailsDTO.setInviteUserAvatar(e.getLogo());
                    detailsDTO.setNickname(e.getName());
                    detailsDTO.setMobile(e.getMobile());
                    userMap.put(e.getId(), detailsDTO);
                });
            }
        } else if(param.getType() == 1){
            List<Long> ids = records.stream().map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<UserDTO> users = this.userService.listUsers(ids);
            if(!CollectionUtils.isEmpty(users)){
                users.forEach(e ->{
                    TeamDetailsDTO detailsDTO = new TeamDetailsDTO();
                    detailsDTO.setInviteUserAvatar(e.getAvatar());
                    detailsDTO.setNickname(e.getNickname());
                    detailsDTO.setMobile(e.getMobile());
                    userMap.put(e.getId(), detailsDTO);
                });
            }
        } else if(param.getType() == 2){
            List<Long> ids = records.stream().map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<MerchantDTO> merchants = this.merchantService.getMerchantByIds(ids);
            if(!CollectionUtils.isEmpty(merchants)) {
                merchants.forEach(e ->{
                    TeamDetailsDTO detailsDTO = new TeamDetailsDTO();
                    detailsDTO.setInviteUserAvatar(e.getLogo());
                    detailsDTO.setNickname(e.getName());
                    detailsDTO.setMobile(e.getMobile());
                    userMap.put(e.getId(), detailsDTO);
                });
            }
        }
        for(TeamDetailsDTO detailsDTO : records){
            Long id = detailsDTO.getInviteUserId();
            TeamDetailsDTO teamDetailsDTO = userMap.get(id);
            if(teamDetailsDTO != null){
                detailsDTO.setName(teamDetailsDTO.getNickname());
                detailsDTO.setNickname(teamDetailsDTO.getNickname());
                detailsDTO.setInviteUserAvatar(teamDetailsDTO.getInviteUserAvatar());
                detailsDTO.setMobile(teamDetailsDTO.getMobile());
            }
            BigDecimal amount = detailsDTO.getAmount();
            if(amount == null){
                detailsDTO.setAmount(BigDecimal.ZERO);
            }
        }
        dto.setRecords(records);
        return dto;
    }

    @Override
    public Long queryUserByTarget(Long userId, Integer type) {
        LambdaQueryWrapper<UserInvite> inviteWrapper = Wrappers.lambdaQuery();
        inviteWrapper.eq(UserInvite::getTargetId, userId);
        inviteWrapper.eq(UserInvite::getType, type);
        inviteWrapper.eq(UserInvite::getStatus, 0);
        UserInvite userInvite = baseMapper.selectOne(inviteWrapper);
        return userInvite == null ? null : userInvite.getUserId();
    }



    @Override
    public TeamDTO inviteList(TeamInvitePageParam param) {
        TeamDTO dto = new TeamDTO();
        Paging paging = param.getPaging();
        int pageNum = (paging.getNumber() - 1) * paging.getSize();
        if(pageNum < 0){
            return dto;
        }
        List<TeamDetailsDTO> records = baseMapper.inviteList(param.getType(), pageNum, paging.getSize());
        if(CollectionUtils.isEmpty(records)){
            return dto;
        }
        List<Long> userList = records.stream().map(TeamDetailsDTO::getUserId).collect(Collectors.toList());
        List<BaseUserDTO> usersLists = this.userService.listBaseInfoUsers(userList);
        Map<Long, String> userMap = usersLists.stream().collect(Collectors.toMap(BaseUserDTO::getId, BaseUserDTO::getNickname));
        //Map<Long, String> mobileMap = usersLists.stream().collect(Collectors.toMap(UserDTO::getId, UserDTO::getNickname));
        Map<Long, String> map = new HashMap<>();
        if(param.getType() == null){
            List<Long> userIds = records.stream().filter(e -> e.getSourceType() == 1).map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<BaseUserDTO> users = this.userService.listBaseInfoUsers(userIds);
            if(!CollectionUtils.isEmpty(users)){
                map = users.stream().collect(Collectors.toMap(BaseUserDTO::getId, BaseUserDTO::getNickname));
            }
            List<Long> merchantIds = records.stream().filter(e -> e.getSourceType() == 2).map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<MerchantDTO> merchants = this.merchantService.getMerchantByIds(merchantIds);
            if(!CollectionUtils.isEmpty(merchants)){
                map.putAll(merchants.stream().collect(Collectors.toMap(MerchantDTO::getId, MerchantDTO::getName)));
            }
        } else if(param.getType() == 1){
            List<Long> ids = records.stream().map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<BaseUserDTO> users = this.userService.listBaseInfoUsers(ids);
            if(!CollectionUtils.isEmpty(users)){
                map = users.stream().collect(Collectors.toMap(BaseUserDTO::getId, BaseUserDTO::getNickname));
            }
        } else if(param.getType() == 2){
            List<Long> ids = records.stream().map(TeamDetailsDTO::getInviteUserId).collect(Collectors.toList());
            List<MerchantDTO> merchants = this.merchantService.getMerchantByIds(ids);
            if(!CollectionUtils.isEmpty(merchants)){
                map = merchants.stream().collect(Collectors.toMap(MerchantDTO::getId, MerchantDTO::getName));
            }
        }
        for(TeamDetailsDTO detailsDTO : records){
            detailsDTO.setName(map.get(detailsDTO.getInviteUserId()));
            detailsDTO.setNickname(userMap.get(detailsDTO.getUserId()));
            BigDecimal amount = detailsDTO.getAmount();
            if(amount == null){
                detailsDTO.setAmount(BigDecimal.ZERO);
            }
        }
        dto = baseMapper.inviteTotalStatistics(param.getType());
        BigDecimal totalAmount = dto.getTotalAmount();
        if(totalAmount == null){
            dto.setTotalAmount(BigDecimal.ZERO);
        }
        dto.setRecords(records);
        return dto;
    }

    @Override
    public void unbind(Long id) {
        UserInvite userInvite = baseMapper.selectById(id);
        if(userInvite == null){
            return;
        }
        userInvite.setStatus(1);
        baseMapper.updateById(userInvite);
    }
}
